/*
 * Picture Puzzle Collection
 * Copyright (c) 1998-1999,2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.pp;

/**
 * Common interface for the various game panels.  The key advantage to this is
 * that the main program only needs an instance of this interface to handle
 * transfering/retrieving information from the various panels.
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
 * CREATED: December 22, 1998
*/
public interface PuzScreen {
  /** Constant indicating that the next mode is the selection screen */
  public static final int SCREEN_SELECTION = 1;

  /** Constant indicating that the next mode is the difficulty screen */
  public static final int SCREEN_DIFFICULTY = 2;

  /** Constant indicating that the next mode is the puzzle screen */
  public static final int SCREEN_PUZZLE = 3;

  /** Constant indicating that the next mode is the sliding screen */
  public static final int SCREEN_SLIDING = 4;

  /** Constant indicating that the next mode is the difficulty screen */
  public static final int SCREEN_ROTATION = 5;

  /** Constant indicating that the next mode is the difficulty screen */
  public static final int SCREEN_GAME = 7;

  /**
    Resets the screen to its starting condition.
  */
  public abstract void reset();


  /**
    Decisecond tick notification.  The controlling object calles this function
    once every decisecond (10th of a second).  It can be used for handling
    animations and other timing dependancies.
  */
  public abstract void tick();


  /**
    Called whenever the mouse is moved so that the screen can handle it if
    it needs to.
  */
  public abstract void mouse_moved(int x, int y);

  /**
    Handle the pressing of the mouse button.  Returns a non-zero value if this
    results in an external action (such as a switch to another screen).
  */
  public abstract int mouse_released(int x, int y);

}

