/*
 * Picture Puzzle Collection
 * Copyright (c) 1998-1999,2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.pp;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.Hashtable;

/**
 * Rotation Puzzle Applet
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
*/
public class RP
extends Applet
implements Runnable, MouseListener, MouseMotionListener {

// *********************
// ***** CONSTANTS *****
// *********************
	private static final long serialVersionUID = 8280320100L;
	
    private static final String PARAM_PLAYFIELD = "PLAYFIELD";
    private static final String PARAM_MOVES = "MOVES";
    private static final String PARAM_PERCENT = "PERCENT";
    private static final String PARAM_TIME = "TIME";

    private static final Rectangle DEF_PLAYFIELD = new Rectangle (210,10,400,400);
    private static final Rectangle DEF_MOVES = new Rectangle (0,10,200,100);
    private static final Rectangle DEF_PERCENT = new Rectangle (0,160,200,80);
    private static final Rectangle DEF_TIME = new Rectangle (0,320,200,80);

// *********************
// ***** VARIABLES *****
// *********************
    
  Thread runner;
  boolean threadAlive = true;

  SelectPanel selectPanel;
  DifficultyPanel difficultyPanel;
  RotationPuzzle picturePuzzle;
  ErrorPanel errorPanel;

  private String errorMessage;
  private boolean errorOccured = false;
  private Hashtable _config = new Hashtable();
  
  PuzScreen screen;

  Image iImages[] = new Image[6];
//  Image iSmallImages[] = new Image[4];
  Image iArrows[] = new Image[4];
  Image iScaledImage;
  private int _puzWidth = 400;
  private int _puzHeight = 400;

  
	public void init() {
		this.addMouseListener(this);
        this.addMouseMotionListener(this);
    
        loadConfig();
        loadImages();
        iScaledImage = createImage(_puzWidth, _puzHeight);

        if (errorOccured)
        {
            errorPanel = new ErrorPanel();
            errorPanel.setError("Problem with Configuration");
            errorPanel.setMessage(errorMessage);
            add("error", errorPanel);
            System.out.println(errorMessage);
        }
        else
        {
            selectPanel = new SelectPanel(this, this);
            selectPanel.setImages("Rotation Puzzle", iImages[0], iImages[1], iImages[2], iImages[3]);
            screen = selectPanel;
            difficultyPanel = new DifficultyPanel(this, this);
        
            picturePuzzle = new RotationPuzzle(this);
            picturePuzzle.setPuzzle(iImages[0], 50, 50);
            picturePuzzle.setArrowImages(iArrows[0], iArrows[1], iArrows[2], iArrows[3]);
            picturePuzzle.setAnimation(4);
            picturePuzzle.setPlayfield(parseRect((String)_config.get(PARAM_PLAYFIELD), DEF_PLAYFIELD));
            picturePuzzle.setCounter(parseRect((String)_config.get(PARAM_MOVES), DEF_MOVES));
            picturePuzzle.setTimer(parseRect((String)_config.get(PARAM_PERCENT), DEF_PERCENT));
            picturePuzzle.setPercent(parseRect((String)_config.get(PARAM_TIME), DEF_TIME));
            
            setLayout(new CardLayout());
            add("select", selectPanel);
            add("difficulty", difficultyPanel);
            add("picture puzzle", picturePuzzle);
            requestFocus();
        }
	}
    // *************************************************
    // ***** METHODS - Configuration File Handling *****
    // *************************************************

    private void loadConfig()
    {
        String s = getParameter("CONFIG");
        if (s == null)
        {
            errorMessage = "Missing properties";
            errorOccured = true;
            return;
        }
        URL url;
        try
        {
            url = new URL(getCodeBase(), s);
        }
        catch (MalformedURLException mue)
        {
            errorMessage = "Invalid URL specified for config";
            errorOccured = true;
            return;
        }

        String temp;
        String command, param;
        int index;
        BufferedReader in;

        try
        {
            in = new BufferedReader(new InputStreamReader(url.openStream()));
            temp = in.readLine();
            do
            {
                //System.out.println(temp);
                index = temp.indexOf(' ');
                if (index > 2) // must have 3 characters before space
                {
                    command = temp.substring(0, index);
                    param = temp.substring(index+1);
                    //System.out.println("CMD=" + command + " param=" + param);
                    _config.put(command, param);
                }
                temp = in.readLine();
            }
            while (temp != null);
            in.close();
        }
        catch (IOException ioe)
        {
            errorMessage = "IO Exception reading config";
            errorOccured = true;
            return;
        }
    }

    public String[] parseString(String s, char ch)
    {
        int cntr, count = 0;
        int len = s.length();
        char sc;

        for (cntr = 0; cntr < len; ++cntr)
        {
            sc = s.charAt(cntr);
            if (sc == ch)
                ++count;
        }

        String rs[] = new String[count+1];
        int start = 0;
        int last = 0;
        for (cntr = 0; cntr < count; ++cntr)
        {
            last = s.indexOf(ch, start);
            rs[cntr] = s.substring(start, last);
            start = last + 1;
        }
        rs[count] = s.substring(start);

        return rs;
    }

    public int parseInt(String s, int d)
    {
        int r;

        try
        {
            r = Integer.parseInt(s);
        }
        catch (Exception e)
        {
            r = d;
        }

        return r;
    }

    public Rectangle parseRect(String s, Rectangle r)
    {
        String args[] = parseString(s, ',');
        if ( (args == null) || (args.length != 4) )
        {
            return r;
        }
        int x = parseInt(args[0], 0);
        int y = parseInt(args[1], 0);
        int w = parseInt(args[2], 0);
        int h = parseInt(args[3], 0);
        return new Rectangle(x, y, w, h);
    }

    public void loadImages()
    {
        int cntr;

        for (cntr = 0; cntr < 6; ++cntr)
        {
            iImages[cntr] = getImage(getCodeBase(), (String) _config.get("PIC["+cntr+"]"));
        }

        iArrows[0] = getImage(getCodeBase(), (String) _config.get("ARROW_UP"));
        iArrows[1] = getImage(getCodeBase(), (String) _config.get("ARROW_RIGHT"));
        iArrows[2] = getImage(getCodeBase(), (String) _config.get("ARROW_DOWN"));
        iArrows[3] = getImage(getCodeBase(), (String) _config.get("ARROW_LEFT"));
        MediaTracker tracker  =  new  MediaTracker(this);
        for (cntr = 0; cntr < 4; ++cntr) {
            tracker.addImage(iArrows[cntr], 1);
        }
        for (cntr = 0; cntr < 6; ++cntr) {
            tracker.addImage(iImages[cntr], 1);
        }
        try  {
            tracker.waitForID(1);
        }  catch  (InterruptedException  e)  {
            System.out.println("Image loading Interrupted Exception");
        }
        if (tracker.isErrorID(1))
        {
            System.out.println("Image loading Error");
            errorMessage = "Image Loading Error";
            errorOccured = true;
        }
    }

    public void start() {
		if (runner == null) {
			runner = new Thread(this);
			runner.start();
		}
	}

	public void run() {
		threadAlive = true;
		while (threadAlive) {
      screen.tick();
			try { Thread.sleep(100); }
  		catch (InterruptedException e) { }
    }
	}

	public void stop() {
		if (runner != null) {
			threadAlive = false;
			runner = null;
		}
	}

	public void destroy() {
		if (runner != null) {
			threadAlive = false;
			runner = null;
		}
    super.destroy();
	}

	public String getAppletInfo() {
    	return "Sliding Puzzle 1.0 OSR. Copyright 1998-1999, 2003-2006 Blazing Games Inc. All Rights Reserved.";
	}

  public synchronized void changeScreens(int id) {
    System.gc();
    if (id == PuzScreen.SCREEN_SELECTION) {
      selectPanel.reset();
      screen = selectPanel;
			((CardLayout)getLayout()).show(this,"select");
    }
    else if (id == PuzScreen.SCREEN_DIFFICULTY) {
      difficultyPanel.reset();
      screen = difficultyPanel;
			((CardLayout)getLayout()).show(this,"difficulty");
    }
    else {
      int dif;
      if (difficultyPanel.getSelected() == 1)
        dif = 133;
      else if (difficultyPanel.getSelected() == 2)
        dif = 100;
      else
        dif = 80;
      Graphics gTemp = iScaledImage.getGraphics();
      if (selectPanel.getSelected() == 3)
          gTemp.drawImage(iImages[2+difficultyPanel.getSelected()], 0,0, _puzWidth, _puzHeight, Color.black, this);
        else
            gTemp.drawImage(iImages[selectPanel.getSelected()], 0,0, _puzWidth, _puzHeight, Color.black, this);

        picturePuzzle.setPuzzle(iScaledImage, dif, dif);
        picturePuzzle.scramble();
      screen = picturePuzzle;
			((CardLayout)getLayout()).show(this,"picture puzzle");
    }
  }

  /**
    Mouse Clicked event from MouseListener.
    Invoked when the mouse has been clicked on a component.
  */
  public void mouseClicked(MouseEvent e) { /* ignore */ }

  /**
    mousePressed event from MouseListener.
    Invoked when a mouse button has been pressed on a component.
  */
  public void mousePressed(MouseEvent e) { /* ignore */ }

 /**
    mouseReleased event from MouseListener.
    Invoked when a mouse button has been released on a component.
  */
  public void mouseReleased(MouseEvent e) {
    int ns = screen.mouse_released(e.getX(), e.getY());
    if (ns > 0)
      changeScreens(ns);
  }

  /**
    mouseEntered event from MouseListener.
    Invoked when the mouse enters a component.
  */
  public void mouseEntered(MouseEvent e) { /* ignore */ }

 /**
    mouseExited event from MouseListener.
    Invoked when the mouse exits a component.
  */
  public void mouseExited(MouseEvent e) { /* ignore */ }

  /**
    mouseDragged event from MouseMotionListener.
    Invoked when a mouse button is pressed on a component and then dragged.
    Mouse drag events will continue to be delivered to the component where the
    first originated until the mouse button is released (regardless of whether
    the mouse position is within the bounds of the component).
  */
  public void mouseDragged(MouseEvent e) { /* ignore */ }

  /**
    mouseMoved event from MouseMotionListener.
    Invoked when the mouse button has been moved on a component (with no buttons no down).
  */
  public void mouseMoved(MouseEvent e) {
    screen.mouse_moved(e.getX(), e.getY());
  }

}
