/*
 * Picture Puzzle Collection
 * Copyright (c) 1998-1999,2003-2006 Blazing Games Inc. All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.blazinggames.pp;

import java.awt.*;
import java.awt.event.*;
import java.util.Random;

/**
 * The Rotation Puzzle game screen
 * 
 * @author Billy D. Spelchan
 * @version 1.0 OSR
*/

public class RotationPuzzle extends PicturePuzzle {

// *** CONSTANTS ***
	private static final long serialVersionUID = 8280710100L;
  // rotation direction constants
  public static final int ROT_UP = 0;
  public static final int ROT_RIGHT = 1;
  public static final int ROT_DOWN = 2;
  public static final int ROT_LEFT = 3;

// *** VARIABLES ***
  /** The images used for drawing the arrows */
  Image iArrows[] = new Image[4];

  /** Animation variables */
  int nAnimRowCol, nAnimDirection;

// *** CONSTRUCTOR ***
	RotationPuzzle(MouseListener ml) {
    super(ml);

    bGridVisible = true;
	}


  /**
    Rotate a row or column.
  */
  public boolean rotate(int direction, int rowcol) {
    int cntr, temp;

    switch (direction) {
      case ROT_UP:
        temp = nPuzzleData[rowcol];
        for (cntr = 0; cntr < (nGridHeight-1); ++cntr)
          nPuzzleData[cntr*nGridWidth+rowcol] =
                nPuzzleData[(cntr+1)*nGridWidth+rowcol];
        nPuzzleData[(nGridHeight-1)*nGridWidth+rowcol] = temp;
        break;
      case ROT_RIGHT:
        temp = nPuzzleData[rowcol*nGridWidth+nGridWidth-1];
        for (cntr = (nGridWidth-1); cntr > 0; --cntr)
          nPuzzleData[rowcol*nGridWidth+cntr] =
                nPuzzleData[rowcol*nGridWidth+cntr-1];
        nPuzzleData[rowcol*nGridWidth] = temp;
        break;
      case ROT_DOWN:
        temp = nPuzzleData[(nGridHeight-1)*nGridWidth+rowcol];
        for (cntr = (nGridHeight-1); cntr > 0 ; --cntr)
          nPuzzleData[cntr*nGridWidth+rowcol] =
                nPuzzleData[(cntr-1)*nGridWidth+rowcol];
        nPuzzleData[rowcol] = temp;
        break;
      case ROT_LEFT:
        temp = nPuzzleData[rowcol*nGridWidth];
        for (cntr = 0; cntr < (nGridWidth-1); ++cntr)
          nPuzzleData[rowcol*nGridWidth+cntr] =
                nPuzzleData[rowcol*nGridWidth+cntr+1];
        nPuzzleData[rowcol*nGridWidth+nGridWidth-1] = temp;
        break;
      default :
        return false;
    }
    nAnimDirection = direction;
    nAnimRowCol = rowcol;
    ++_counterValue;
    return true;
  }

  /**
    Sets the arrow images to use.
  */
  public void setArrowImages(Image iU, Image iR, Image iD, Image iL) {
    iArrows[0] = iU;
    iArrows[1] = iR;
    iArrows[2] = iD;
    iArrows[3] = iL;
  }

  /**
    Mixes up the puzzle.
  */
  public void scramble() {
    int nNumMoves = nGridWidth * nGridHeight * 10;
    int cntr, moveDir, temp;
    Random r = new Random();

    for (cntr = 0; cntr < nNumMoves; ++cntr) {
        moveDir = Math.abs(r.nextInt()) % 4;
        if ((moveDir == ROT_RIGHT) || (moveDir == ROT_LEFT))
          temp = Math.abs(r.nextInt()) % nGridHeight;
        else
          temp = Math.abs(r.nextInt()) % nGridWidth;
        rotate(moveDir, temp);
    }
    hasWon = false;
    nCurrentSelection = -1;
    _counterValue = 0;
    _startTime = System.currentTimeMillis();
  }

  /**
    Paints the animation.
  */
  synchronized void paintAnimation(Graphics g) {
    int nNumFrames, cellID;

    if ( (nAnimDirection == ROT_UP) || (nAnimDirection == ROT_DOWN) )
      nNumFrames = nGridPixelsY;
    else
      nNumFrames = nGridPixelsX;

    nAnimationFrame += nAnimationRate;
    if (nAnimationFrame >= nNumFrames) {
      animationPlaying = false;
      return;
    }

    int nImWidth = iPuzzle.getWidth(this);
    int nImHeight = iPuzzle.getHeight(this);
    int PicLeft = (getSize().width - nImWidth) / 2;
    int PicTop = (getSize().height -  nImHeight) / 2;
    if (_usePlayfield)
    {
        PicLeft = _playfieldRect.x;
        PicTop = _playfieldRect.y;
    }
    
    switch (nAnimDirection) {
      case ROT_UP:
        g.copyArea(PicLeft + nPicLeftGap + nAnimRowCol * nGridPixelsX,
              PicTop + nPicTopGap + nAnimationRate,
              nGridPixelsX, nGridPixelsY * nGridHeight - nAnimationRate,
              0, -nAnimationRate);
        cellID = nPuzzleData[nGridWidth * (nGridHeight-1) + nAnimRowCol];
        g.drawImage(iPuzzle, PicLeft + nPicLeftGap + nAnimRowCol * nGridPixelsX,
              PicTop + nPicTopGap + nGridHeight * nGridPixelsY - nAnimationFrame,
              PicLeft + nPicLeftGap + nAnimRowCol * nGridPixelsX + nGridPixelsX,
              PicTop + nPicTopGap + nGridHeight * nGridPixelsY,
              nPicLeftGap + (cellID % nGridWidth) * nGridPixelsX,
              nPicTopGap + (cellID / nGridWidth) * nGridPixelsY,
              nPicLeftGap + (cellID % nGridWidth) * nGridPixelsX + nGridPixelsX,
              nPicTopGap + (cellID / nGridWidth) * nGridPixelsY + nAnimationFrame,
              this);
        break;
      case ROT_RIGHT:
        g.copyArea(PicLeft + nPicLeftGap,
              PicTop + nPicTopGap + nAnimRowCol * nGridPixelsY,
              nGridPixelsX * nGridWidth - nAnimationRate, nGridPixelsY,
              nAnimationRate, 0);
        cellID = nPuzzleData[nGridWidth * nAnimRowCol];
        g.drawImage(iPuzzle, PicLeft + nPicLeftGap,
              PicTop + nPicTopGap + nGridPixelsY * nAnimRowCol,
              PicLeft + nPicLeftGap + nAnimationFrame,
              PicTop + nPicTopGap + nGridPixelsY * nAnimRowCol + nGridPixelsY,
              nPicLeftGap + (cellID % nGridWidth) * nGridPixelsX + nGridPixelsX - nAnimationFrame,
              nPicTopGap + (cellID / nGridWidth) * nGridPixelsY,
              nPicLeftGap + (cellID % nGridWidth) * nGridPixelsX + nGridPixelsX,
              nPicTopGap + (cellID / nGridWidth) * nGridPixelsY + nGridPixelsY,
              this);
        break;
      case ROT_DOWN:
        g.copyArea(PicLeft + nPicLeftGap + nAnimRowCol * nGridPixelsX,
              PicTop + nPicTopGap,
              nGridPixelsX, nGridPixelsY * nGridHeight - nAnimationRate,
              0, nAnimationRate);
        g.drawImage(iPuzzle, PicLeft + nPicLeftGap + nAnimRowCol * nGridPixelsX,
              PicTop + nPicTopGap,
              PicLeft + nPicLeftGap + nAnimRowCol * nGridPixelsX + nGridPixelsX,
              PicTop + nPicTopGap + nAnimationFrame,
              nPicLeftGap + (nPuzzleData[nAnimRowCol] % nGridWidth) * nGridPixelsX,
              nPicTopGap + (nPuzzleData[nAnimRowCol] / nGridWidth) * nGridPixelsY + nGridPixelsY - nAnimationFrame,
              nPicLeftGap + (nPuzzleData[nAnimRowCol] % nGridWidth) * nGridPixelsX + nGridPixelsX,
              nPicTopGap + (nPuzzleData[nAnimRowCol] / nGridWidth) * nGridPixelsY + nGridPixelsY,
              this);
        break;
      case ROT_LEFT:
        g.copyArea(PicLeft + nPicLeftGap + nAnimationRate,
              PicTop + nPicTopGap + nAnimRowCol * nGridPixelsY,
              nGridPixelsX * nGridWidth - nAnimationRate, nGridPixelsY,
              -nAnimationRate, 0);
        cellID = nPuzzleData[nGridWidth * nAnimRowCol + nGridWidth-1];
        g.drawImage(iPuzzle, PicLeft + nPicLeftGap + nGridPixelsX * nGridWidth - nAnimationFrame,
              PicTop + nPicTopGap + nGridPixelsY * nAnimRowCol,
              PicLeft + nPicLeftGap + nGridPixelsX * nGridWidth,
              PicTop + nPicTopGap + nGridPixelsY * nAnimRowCol + nGridPixelsY,
              nPicLeftGap + (cellID % nGridWidth) * nGridPixelsX,
              nPicTopGap + (cellID / nGridWidth) * nGridPixelsY,
              nPicLeftGap + (cellID % nGridWidth) * nGridPixelsX + nAnimationFrame,
              nPicTopGap + (cellID / nGridWidth) * nGridPixelsY + nGridPixelsY,
              this);
        break;
    }
  }


  /**
    paints the image and (optionally) the grid.
  */
	public void paint(Graphics g) {
    if (iPuzzle == null)
      return;

    super.paint(g);

    int nImWidth = iPuzzle.getWidth(this);
    int nImHeight = iPuzzle.getHeight(this);
    int PicLeft = (getSize().width - nImWidth) / 2;
    int PicTop = (getSize().height -  nImHeight) / 2;
    if (_usePlayfield)
    {
        PicLeft = _playfieldRect.x;
        PicTop = _playfieldRect.y;
    }
    
    // Now paint the arrows if the game isn't over
    if ((!hasWon) && (!animationPlaying)) {
      int cntr, nTemp;

      for (cntr = 0; cntr < nGridWidth; ++cntr) {
        nTemp = PicLeft + nPicLeftGap + cntr * nGridPixelsX + (nGridPixelsX >> 1);
        g.drawImage(iArrows[0], nTemp, PicTop - 10, this);
        g.drawImage(iArrows[2], nTemp, PicTop + nImHeight, this);
      }
      for (cntr = 0; cntr < nGridHeight; ++cntr) {
        nTemp = PicTop + nPicTopGap +cntr * nGridPixelsY + (nGridPixelsY >> 1);
        g.drawImage(iArrows[3], PicLeft - 10, nTemp, this);
        g.drawImage(iArrows[1], PicLeft + nImWidth, nTemp, this);
      }
    }
    else { // Otherwise erase the arrows
      g.setColor(Color.white);
      g.fillRect(PicLeft, 0, nImWidth, PicTop);
      g.fillRect(PicLeft, PicTop + nImHeight, nImWidth, PicTop);
      g.fillRect(0, PicTop, PicLeft, nImHeight);
      g.fillRect(PicLeft + nImWidth, PicTop, PicLeft, nImHeight);
    }
	}


  /**
    Handle the pressing of the mouse button.  Returns a non-zero value if this
    results in an external action (such as a switch to another screen). The
    external action values are described in the MWAGConst static class.
  */
  public int mouse_released(int x, int y) {
    if (animationPlaying)
      return 0;

    if (hasWon)
      return PuzScreen.SCREEN_SELECTION;

    int gx, gy;
    int nImWidth = iPuzzle.getWidth(this);
    int nImHeight = iPuzzle.getHeight(this);
    int PicLeft = (getSize().width - nImWidth) / 2;
    int PicTop = (getSize().height -  nImHeight) / 2;
    if (_usePlayfield)
    {
        PicLeft = _playfieldRect.x;
        PicTop = _playfieldRect.y;
    }
    
    if (x <= PicLeft)
      gx = -1;
    else
      gx = (x - PicLeft - nPicLeftGap) / nGridPixelsX;

    if (y <= PicTop)
      gy = -1;
    else
      gy = (y - PicTop - nPicTopGap) / nGridPixelsY;

    if ( (gx >= 0) || (gx < nGridWidth) ) {
      if (gy < 0) {
        rotate(ROT_UP, gx);
        if (nAnimationRate > 0) {
          nAnimationFrame = 0;
          animationPlaying = true;
        }
      }
      else if (gy >= nGridHeight) {
        rotate(ROT_DOWN, gx);
        if (nAnimationRate > 0) {
          nAnimationFrame = 0;
          animationPlaying = true;
        }
      }
    }

    if ( (gy >= 0) || (gy < nGridHeight) ) {
      if (gx < 0) {
        rotate(ROT_LEFT, gy);
        if (nAnimationRate > 0) {
          nAnimationFrame = 0;
          animationPlaying = true;
        }
      }
      else if (gx >= nGridWidth) {
        rotate(ROT_RIGHT, gy);
        if (nAnimationRate > 0) {
          nAnimationFrame = 0;
          animationPlaying = true;
        }
      }
    }

    hasWon = isWon();

    repaint();
    return 0;
  }

}
